import six

BOOL_TRUEISH = ['1', 'yes', 'y', 't', 'true', 'on', 'enabled']


def as_bool(value, default=False):
    if isinstance(value, bool):
        return value
    if value is None:
        return default
    value = six.text_type(value)
    value = value.strip().lower()
    if not len(value):
        return default
    return value in BOOL_TRUEISH



from collections import Mapping

from banal.lists import is_sequence


def is_mapping(obj):
    return isinstance(obj, Mapping)


def clean_dict(data):
    """Remove None-valued keys from a dictionary, recursively."""
    if is_mapping(data):
        out = {}
        for k, v in data.items():
            if v is not None:
                out[k] = clean_dict(v)
        return out
    elif is_sequence(data):
        return [clean_dict(d) for d in data if d is not None]
    return data



import six
import types
from itertools import chain
from hashlib import sha1
from datetime import date, datetime

from banal.dicts import is_mapping
from banal.lists import is_sequence


def bytes_iter(obj):
    """Turn a complex object into an iterator of byte strings.
    The resulting iterator can be used for caching.
    """
    if obj is None:
        return
    elif isinstance(obj, six.binary_type):
        yield obj
    elif isinstance(obj, six.text_type):
        yield obj
    elif isinstance(obj, (date, datetime)):
        yield obj.isoformat()
    elif is_mapping(obj):
        for key in sorted(obj.keys()):
            for out in chain(bytes_iter(key), bytes_iter(obj[key])):
                yield out
    elif is_sequence(obj):
        if isinstance(obj, set):
            obj = sorted(obj)
        for item in obj:
            for out in bytes_iter(item):
                yield out
    elif isinstance(obj, (types.FunctionType, types.BuiltinFunctionType,
                          types.MethodType, types.BuiltinMethodType,
                          types.UnboundMethodType)):
        yield getattr(obj, 'func_name', '')
    else:
        yield unicode(obj)


def hash_data(obj):
    """Generate a SHA1 from a complex object."""
    collect = sha1()
    for text in bytes_iter(obj):
        if isinstance(obj, six.text_type):
            text = text.encode('utf-8')
        collect.update(text)
    return collect.hexdigest()



import six
import sys


def decode_path(file_path):
    """Turn a path name into unicode."""
    if file_path is None:
        return
    if isinstance(file_path, six.binary_type):
        file_path = file_path.decode(sys.getfilesystemencoding())
    return file_path



import six
from collections import Sequence


def is_sequence(obj):
    """Check if something quacks like a list."""
    return isinstance(obj, Sequence) and not isinstance(obj, six.string_types)


def unique_list(lst):
    """Make a list unique, retaining order of initial appearance."""
    uniq = []
    for item in lst:
        if item not in uniq:
            uniq.append(item)
    return uniq


def ensure_list(obj):
    """Make the returned object a list, otherwise wrap as single item."""
    if obj is None:
        return []
    if not is_sequence(obj):
        return [obj]
    return obj


def first(lst):
    """Return the first non-null element in the list, or None."""
    for item in ensure_list(lst):
        if item is not None:
            return item



from banal.lists import is_sequence, ensure_list, unique_list
from banal.lists import first
from banal.dicts import is_mapping, clean_dict
from banal.filesystem import decode_path
from banal.cache import hash_data
from banal.bools import as_bool

__all__ = [is_sequence, ensure_list, unique_list, first,
           as_bool,
           is_mapping, clean_dict,
           decode_path, hash_data]
